#!/jwst/bin/python
# Copyright (C) 2010 Association of Universities for Research in Astronomy(AURA)
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
# 
#     1. Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
# 
#     2. Redistributions in binary form must reproduce the above
#       copyright notice, this list of conditions and the following
#       disclaimer in the documentation and/or other materials provided
#       with the distribution.
# 
#     3. The name of AURA and its representatives may not be used to
#       endorse or promote products derived from this software without
#       specific prior written permission.
# 
# THIS SOFTWARE IS PROVIDED BY AURA ``AS IS'' AND ANY EXPRESS OR IMPLIED
# WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL AURA BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
# OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
# TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
# USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
# DAMAGE.
"""
Pushes resulting ACS science data products into iRODS.
Uses the following directory convention for storing the results:
ippp/ss/dataFileName

First attempts to create the ippp and ss dirs, in case they don't already exist.
For an ASN, pushes all the ASN products, then member products.
For a singleton, just pushes the singleton products.

iRODS URIs have to have the form
    irods://user.zone:passwrod@server:port/path/to/file/or/collection

file URIs have to have the form
    [file:]/path/to/file
"""
import irods_plus
import opusutil
import pyfits
import opusutil_pyfits

class NoFilesFound(Exception):
  pass


def isASN(datasetName):
  if datasetName[-1:]=='0': return True
  else:                     return False

def getProductRoots(localDir, datasetName):
  # assumes ASN, opens ASN table, reads member entries to form list
  asn_filename = localDir + os.sep + datasetName + "_asn.fits"
  opusutil.PrintMsg("I","reading ASN file: "+asn_filename)
  asn_members = opusutil_pyfits.get_asn_members(asn_filename)
  #
  # add on product root (ipppssNN, e.g. j9qo0201)
  asn_members.append(datasetName[:8])
  # comes back in mixed case
  return asn_members

if(__name__ == '__main__'):
    import os
    import glob
    import sys
    import string
    
    # datasetName,directory come in via env vars
    datasetName = os.environ['OSF_DATASET']
    localDir    = os.environ['INPATH']

    # grab ippp and ss from the datasetName
    ippp = datasetName[:4]
    ss   = datasetName[4:6]

    # make iRODS calls to create dirs 
    # (ignore errors, since it likely means the dirs already exist)
    #
    # FOR NOW, hard-coded iRODS paths, since this won't be a permanent part
    #  of the processing
    #
    base_uri = 'irods://dadsops.hlspZone:irods@dmslab1:1247/hlspZone/home/rods/dads/pipeline/outputs/'
    dst_uri  = base_uri + ippp + os.sep
    opusutil.PrintMsg("I","calling imdir for "+dst_uri)
    irods_plus.mkdir(dst_uri)

    dst_uri  += ss + os.sep
    opusutil.PrintMsg("I","calling imdir for "+dst_uri)
    irods_plus.mkdir(dst_uri)
    datasetBase_uri = dst_uri    # save for later use

    # obtain list of product file root names
    if (isASN(datasetName)):
       opusutil.PrintMsg("I","calling getProductRoots...")
       productRoots = getProductRoots(localDir, datasetName)
    else: # singleton
       productRoots = [datasetName]
    opusutil.PrintMsg("I","productRoots= "+str(productRoots))

    # locate files
    for root in productRoots:
        lroot = string.lower(root)
        # grab ALL files (.fits, .tra, anything else with the rootname)
        filemask = localDir + os.sep + lroot + '*'
        files = glob.glob(filemask)
        if not files:
           opusutil.printMsg("E","No Files Found for "+filemask)
           raise NoFilesFound
        for file in files:
           # obtain filename,ext from full filename and add to base URI
           basefile = os.path.basename(file)
           dst_uri = datasetBase_uri + basefile
           # PUSH the file to iRODS
           opusutil.PrintMsg("I","PUSHING: "+file+" TO "+dst_uri)
           irods_plus.put(file, dst_uri)

    #
    sys.exit(0)
